// ---------------------------------------------------------------------------
// scavenger.cs -- Version 2.5 -- January 8, 2001
// by Lorne Laliberte (writer@planetstarsiege.com)
//
// Automatically drop unwanted items you pick up in the field
//
// http://www.writerscripts.com
// ---------------------------------------------------------------------------

include("writer\\version.cs");
version("writer\\scavenger.cs", "2.5", "Lorne Laliberte", "Jan 8, 2001");

include("writer\\event.cs");
include("writer\\fast_faves.cs");
include("writer\\inventory.cs");
include("writer\\item_events.cs");
include("writer\\modifier.cs");
include("writer\\save.cs");
include("writer\\schedule.cs");
include("writer\\whatif.cs");
include("writer\\wtoggle.cs");

// ---------------------------------------------------------------------------
// DEFAULT SETTINGS:
//
// Edit these if you're running this script for the first time
// All subsequent settings must be done in the config\wp_scavenger.cs file

    // Key that will override the auto-drop action when held down
    // I recommend using a modifier like "lshift" or "control" for this
    $Scavenger::OverrideHoldKey = "";

    // Automatically drop packs that aren't in the fast_faves loadout
    // you most recently requested
    $Scavenger::dropPacks = "false";

    // Automatically drop weapons that aren't in the fast_faves loadout
    // you most recently requested
    $Scavenger::dropWeapons = "true";

    // Automatically drop ammo for any weapons you don't have
    $Scavenger::dropAmmo = "false";

    // Number of weapon slots you want to keep free while scavenging
    // Set this to -1 to only keep weapons in your current loadout, EVER
    $Scavenger::FreeWeaponSlots = 1;

    // List of weapons to drop in descending order -- from the weapon you
    // want to drop the most, to the weapon you want to drop the least
    // Note: any weapons not in the list won't be dropped

    Scavenger::DropPriority("Blaster");
    Scavenger::DropPriority("ELF gun");
    Scavenger::DropPriority("Chaingun");
    Scavenger::DropPriority("Grenade Launcher");
    Scavenger::DropPriority("Plasma Gun");
    Scavenger::DropPriority("Mortar");
    Scavenger::DropPriority("Laser Rifle");
    Scavenger::DropPriority("Disc Launcher");

    // Automatically free up slots (i.e. drop one or more weapons) at the start of a match
    $Scavenger::onMatchStart = "false";

    // Automatically free up slots (i.e. drop one or more weapons) when you respawn
    $Scavenger::onRespawn = "false";

    // Automatically free up slots (i.e. drop one or more weapons) when you pick up a pack
    $Scavenger::onPackFound = "false";

    // Toggle between Auto Drop modes
    //bind(keyboard0, make, alt, "a", TO, "Scavenger::Toggle();");
    //bind(keyboard0, break, alt, "a", TO, "", Scavenger::Toggle_break);

    // Manually drop unwanted weapons until the specified number of
    // weapons slots are empty
    //bind(keyboard0, make, alt, "e", TO, "Scavenger::empty(1);", Scavenger::empty_make);
    //bind(keyboard0, break, alt, "e", TO, "", Scavenger::empty_break);

    // Manually drop all unwanted weapons
    //bind(keyboard0, make, alt, "d", TO, "Scavenger::dropAllUnwantedWeapons();");
    //bind(keyboard0, break, alt, "d", TO, "", Scavenger::dropAllUnwantedWeapons_break);

// ---------------------------------------------------------------------------

WriterPrefs::load("scavenger");


if($Scavenger::onMatchStart)
    include("writer\\you_spawned.cs");

if($Scavenger::onRespawn)
    include("writer\\you_respawned.cs");


function Scavenger::Toggle::onExit(%hilite)
{
    $Scavenger::dropPacks   = $Scavenger::ToggleArray[0];
    $Scavenger::dropWeapons = $Scavenger::ToggleArray[1];
    $Scavenger::dropAmmo    = $Scavenger::ToggleArray[2];

    $Scavenger::Toggle::hilite = %hilite;

    // Restore the scavenger::toggle key bind
    editActionMap($Bind::map["Scavenger::Toggle();"]);
    eval(bind::oldString("Scavenger::Toggle();"));
    return;
}

$Scavenger::Toggle::hilite = 0;

function Scavenger::Toggle()
{
    $Scavenger::ToggleArray[0] = $Scavenger::dropPacks;
    $Scavenger::ToggleArray[1] = $Scavenger::dropWeapons;
    $Scavenger::ToggleArray[2] = $Scavenger::dropAmmo;

    // Temporarily disable the scavenger::toggle key bind
    editActionMap($Bind::map["Scavenger::Toggle();"]);
    eval(bind::oldString("Scavenger::Toggle();", "wtoggle::cancel();"));

    wtoggle::reset();
    wtoggle::addItem("Drop Unwanted Packs");
    wtoggle::addItem("Drop Unwanted Weapons");
    wtoggle::addItem("Drop Unwanted Ammo");
    wtoggle::make("Scavenger Toggles\n<f0>(hit ESC when done)\n", "$Scavenger::ToggleArray", 0, "Scavenger::Toggle::onExit(%1);", 3, $Scavenger::Toggle::hilite, false, false);
}

function Scavenger::onWeaponReceived(%itemtype, %itemname, %number, %msg)
{
    if(!$Scavenger::dropWeapons)
        return;

    if($Modifier::Pressed[$Scavenger::OverrideHoldKey])
        return;

    Scavenger::process($Scavenger::FreeWeaponSlots, %itemname);

    // call process again with the %scheduled flag set, to ensure we keep a slot free if we picked
    // a lot of stuff up at once (which can sometimes leave us with an unwanted item)
    schedule::add("Scavenger::process();", 1, scavenger_process);
    return;
}

function Scavenger::empty(%slots)
{
    Scavenger::process(%slots);
}

function Scavenger::dropAllUnwantedWeapons()
{
    Scavenger::process(-1);
}

function Scavenger::process(%free_weapon_slots, %item_picked_up)
{
    if(!$Scavenger::dropWeapons && (%item_picked_up != ""))
        return;

    if(%free_weapon_slots == "")
        %free_weapon_slots = $Scavenger::FreeWeaponSlots;

    if(%free_weapon_slots > -1)
    {
        // figure out how many free weapon slots we have left:

        %max_weapons = $Inv::maxWeapons[loadout::armor()];

        loadout::weapons();

        if(%max_weapons == "")
        {
            %slots_free = 0;
            %free_slots_wanted = 1;
        }
        else
        {
            if(%item_picked_up == "")
                %slots_free = $Inv::maxWeapons[loadout::armor()] - ($loadout::weaponCount);
            else
                %slots_free = $Inv::maxWeapons[loadout::armor()] - ($loadout::weaponCount + 1); // add one for the item we just picked up since it isn't visible to getItemCount yet

            // don't drop anything if we still have as many or more free slots than we want to keep free
            if(%slots_free >= %free_weapon_slots)
                return;

            %free_slots_wanted = %free_weapon_slots;
        }
    }

    %currentweapon = getMountedItem(0);

    for(%i = 1; %i < 7; %i++)
    {
        if($FastFaves::weapon[%i] == "")
            continue;

        %wanted[$FastFaves::weapon[%i]] = true;
    }

    for(%i = 0; %i < $Scavenger::DropPriorityCount; %i++)
    {
        if(getItemCount($Scavenger::DropPriority[%i]) || !String::ICompare($Scavenger::DropPriority[%i], %item_picked_up) )
        {
            if(%wanted[$Scavenger::DropPriority[%i]])
                continue; // don't drop a weapon we want

            if( (%free_weapon_slots == -1) || (%slots_free < %free_slots_wanted) )
            {
                drop($Scavenger::DropPriority[%i]);
                %slots_free++;

                if($Inv::Type[$Scavenger::DropPriority[%i]] == %currentweapon)
                {
                    %currentweapon = -1;
                    nextWeapon();
                }
            }
        }
    }

    if(%currentweapon != -1)
        useType(%currentweapon);
}

function Scavenger::onAmmoReceived(%itemtype, %itemname, %number, %msg)
{
    if(!$Scavenger::dropAmmo)
        return;

    if($Modifier::Pressed[$Scavenger::OverrideHoldKey])
        return;

    // check whether the current loadout lists this ammo as an extra item
    if(String::findSubStr($FastFaves::extraString, %itemname) != -1)
        return; // the loadout wants this ammo whether we have a weapon for it or not

    loadout::weapons(); // update current weapons list

    for(%i = 1; %i <= $loadout::weaponCount; %i++)
    {
        if($Inv::Type[$Inv::Ammo[$loadout::weapon[%i]]] == %itemtype) // we need this ammo
            return;
    }

    // if we made it this far, none of the weapons we're carrying need the ammo, so drop it
    for(%i = 0; %i < 20; %i++)
        dropType(%itemtype);

    return;
}

function Scavenger::onPackReceived(%itemtype, %itemname, %number, %msg)
{
    if(!$Scavenger::dropPacks)
    {
        if($Scavenger::onPackFound)
            Scavenger::process();

        return;
    }

    if($Modifier::Pressed[$Scavenger::OverrideHoldKey])
        return;

    if(String::ICompare($FastFaves::Pack, %itemname)) // $FastFaves::Pack != %itemname
        dropType(%itemtype);

    if($Scavenger::onPackFound)
        Scavenger::process();

    return;
}

function Scavenger::onToolReceived(%itemtype, %itemname, %number, %msg)
{
    if(!$Scavenger::dropWeapons)
        return;

    if($Modifier::Pressed[$Scavenger::OverrideHoldKey])
        return;

    if($Scavenger::keepMountedWeapon)
    {
        if(getMountedItem(0) == %itemtype)
            return;
    }

    if(%itemtype == $Inv::Targeting_Laser)
    {
        if(!$FastFaves::tlaser)
            dropType(%itemtype);
    }
    return;
}

function Scavenger::onThrowableReceived(%itemtype, %itemname, %number, %msg)
{
    if($Modifier::Pressed[$Scavenger::OverrideHoldKey])
        return;

    if(%itemtype == $Inv::Grenade)
    {
        if(!$FastFaves::grenades)
        {
            for(%i = 0; %i < 10; %i++)
                dropType(%itemtype);
        }
    }

    if(%itemtype == $Inv::Mine)
    {
        if(!$FastFaves::mines)
        {
            for(%i = 0; %i < 10; %i++)
                dropType(%itemtype);
        }
    }
    return;
}

function Scavenger::onMatchStart()
{
    if($Scavenger::onMatchStart)
        schedule("Scavenger::process(\"\", 1);", 1);

    return;
}

function Scavenger::onRespawn()
{
    if($Scavenger::onRespawn)
        schedule("Scavenger::process(\"\", 1);", 1);

    return;
}

function scavenger::save()
{
    AddExportText("    // Key that will override the auto-drop action when held down");
    AddExportText("    // I recommend using a modifier like \"lshift\" or \"control\" for this");
    ExportSetting("    $Scavenger::OverrideHoldKey", "");
    AddExportText("");
    AddExportText("    // Automatically drop packs that aren't in the fast_faves loadout");
    AddExportText("    // you most recently requested");
    ExportSetting("    $Scavenger::dropPacks", false);
    AddExportText("");
    AddExportText("    // Automatically drop weapons that aren't in the fast_faves loadout");
    AddExportText("    // you most recently requested");
    ExportSetting("    $Scavenger::dropWeapons", true);
    AddExportText("");
    AddExportText("    // Automatically drop ammo for any weapons you don't have");
    ExportSetting("    $Scavenger::dropAmmo", false);
    AddExportText("");
    AddExportText("    // Number of weapon slots you want to keep free while scavenging");
    AddExportText("    // Set this to -1 to only keep weapons in your current loadout, EVER");
    ExportSetting("    $Scavenger::FreeWeaponSlots", 1);
    AddExportText("");
    AddExportText("    // List of weapons to drop in descending order -- from the weapon you");
    AddExportText("    // want to drop the most, to the weapon you want to drop the least");
    AddExportText("    // Note: any weapons not in the list won't be dropped");
    AddExportText("");

    if($Scavenger::DropPriorityCount)
    {
        for(%i = 0; %i < $Scavenger::DropPriorityCount; %i++)
        {
            AddExportText("    Scavenger::DropPriority(\"" @ $Scavenger::DropPriority[%i] @ "\");");
        }
    }
    else
    {
        AddExportText("    Scavenger::DropPriority(\"Blaster\"); // drop first");
        AddExportText("    Scavenger::DropPriority(\"ELF gun\");");
        AddExportText("    Scavenger::DropPriority(\"Chaingun\");");
        AddExportText("    Scavenger::DropPriority(\"Grenade Launcher\");");
        AddExportText("    Scavenger::DropPriority(\"Plasma Gun\");");
        AddExportText("    Scavenger::DropPriority(\"Mortar\");");
        AddExportText("    Scavenger::DropPriority(\"Laser Rifle\");");
        AddExportText("    Scavenger::DropPriority(\"Disc Launcher\"); // drop last");
    }

    AddExportText("");
    AddExportText("    // Automatically free up slots (i.e. drop one or more weapons) at the start of a match");
    ExportSetting("    $Scavenger::onMatchStart", "false");
    AddExportText("");
    AddExportText("    // Automatically free up slots (i.e. drop one or more weapons) when you respawn");
    ExportSetting("    $Scavenger::onRespawn", "false");
    AddExportText("");
    AddExportText("    // Automatically free up slots (i.e. drop one or more weapons) when you pick up a pack");
    ExportSetting("    $Scavenger::onPackFound", "false");

    AddExportText("");
    AddExportText("    // Toggle between Auto Drop modes");

    ExportBind("Scavenger::Toggle();", "//bind(keyboard0, make, alt, \"a\", TO, \"Scavenger::Toggle();\");");
    ExportBind("Scavenger::Toggle_break", "//bind(keyboard0, break, alt, \"a\", TO, \"\", Scavenger::Toggle_break);");

    AddExportText("");
    AddExportText("    // Manually drop unwanted weapons until the specified number of");
    AddExportText("    // weapons slots are empty");

    ExportBind("Scavenger::empty_make", "//bind(keyboard0, make, alt, \"e\", TO, \"Scavenger::empty(1);\", Scavenger::empty_make);");
    ExportBind("Scavenger::empty_break","//bind(keyboard0, break, alt, \"e\", TO, \"\", Scavenger::empty_break);");

    AddExportText("");
    AddExportText("    // Manually drop all unwanted weapons");

    ExportBind("Scavenger::dropAllUnwantedWeapons();", "//bind(keyboard0, make, alt, \"d\", TO, \"Scavenger::dropAllUnwantedWeapons();\");");
    ExportBind("Scavenger::dropAllUnwantedWeapons_break","//bind(keyboard0, break, alt, \"d\", TO, \"\", Scavenger::dropAllUnwantedWeapons_break);");
}

WriterPrefs::save("scavenger");

function scavenger::init()
{
    needs("writer\\scavenger.cs", "writer\\event.cs 2.7",
                                  "writer\\fast_faves.cs 7.6",
                                  "writer\\inventory.cs 3.3",
                                  "writer\\modifier.cs 2.5",
                                  "writer\\save.cs 2.3",
                                  "writer\\schedule.cs 1.3",
                                  "writer\\whatif.cs 1.1",
                                  "writer\\wtoggle.cs 1.1");

    if($Scavenger::OverrideHoldKey != "")
        Modifier::bind($Scavenger::OverrideHoldKey);

    if($Scavenger::onMatchStart)
    {
        needs("writer\\scavenger.cs", "writer\\you_spawned.cs 1.1");
        Event::Attach(eventYouSpawned, "Scavenger::onMatchStart();");
        MsgEvent::Attach("Match Started.", "Scavenger::onMatchStart();");
    }

    if($Scavenger::onRespawn)
    {
        needs("writer\\scavenger.cs", "writer\\you_respawned.cs 1.2");
        Event::Attach(eventYouRespawned, "Scavenger::onRespawn();");
    }

    Event::Attach(eventWeaponReceived, Scavenger::onWeaponReceived);
    Event::Attach(eventToolReceived, Scavenger::onToolReceived);
    Event::Attach(eventPackReceived, Scavenger::onPackReceived);
    Event::Attach(eventAmmoReceived, Scavenger::onAmmoReceived);
    Event::Attach(eventThrowableReceived, Scavenger::onThrowableReceived);

}
Event::Attach(writerInit, "scavenger::init();");

if(!$UsingWriterLoader) scavenger::init();
